@@ -14,7 +14,7 @@
 /*--------------------------------------------------------------------------*/
 int ffuky( fitsfile *fptr,     /* I - FITS file pointer        */
            int  datatype,      /* I - datatype of the value    */
-           char *keyname,      /* I - name of keyword to write */
+           const char *keyname,      /* I - name of keyword to write */
            void *value,        /* I - keyword value            */
            char *comm,         /* I - keyword comment          */
            int  *status)       /* IO - error status            */
@@ -32,23 +32,23 @@ int ffuky( fitsfile *fptr,     /* I - FITS file pointer        */
     }
     else if (datatype == TBYTE)
     {
-        ffukyj(fptr, keyname, (long) *(unsigned char *) value, comm, status);
+        ffukyj(fptr, keyname, (LONGLONG) *(unsigned char *) value, comm, status);
     }
     else if (datatype == TSBYTE)
     {
-        ffukyj(fptr, keyname, (long) *(signed char *) value, comm, status);
+        ffukyj(fptr, keyname, (LONGLONG) *(signed char *) value, comm, status);
     }
     else if (datatype == TUSHORT)
     {
-        ffukyj(fptr, keyname, (long) *(unsigned short *) value, comm, status);
+        ffukyj(fptr, keyname, (LONGLONG) *(unsigned short *) value, comm, status);
     }
     else if (datatype == TSHORT)
     {
-        ffukyj(fptr, keyname, (long) *(short *) value, comm, status);
+        ffukyj(fptr, keyname, (LONGLONG) *(short *) value, comm, status);
     }
     else if (datatype == TINT)
     {
-        ffukyj(fptr, keyname, (long) *(int *) value, comm, status);
+        ffukyj(fptr, keyname, (LONGLONG) *(int *) value, comm, status);
     }
     else if (datatype == TUINT)
     {
@@ -66,7 +66,11 @@ int ffuky( fitsfile *fptr,     /* I - FITS file pointer        */
     }
     else if (datatype == TLONG)
     {
-        ffukyj(fptr, keyname, *(long *) value, comm, status);
+        ffukyj(fptr, keyname, (LONGLONG) *(long *) value, comm, status);
+    }
+    else if (datatype == TLONGLONG)
+    {
+        ffukyj(fptr, keyname, *(LONGLONG *) value, comm, status);
     }
     else if (datatype == TFLOAT)
     {
@@ -91,7 +95,7 @@ int ffuky( fitsfile *fptr,     /* I - FITS file pointer        */
 } 
 /*--------------------------------------------------------------------------*/
 int ffukyu(fitsfile *fptr,    /* I - FITS file pointer  */
-           char *keyname,     /* I - keyword name       */
+           const char *keyname,     /* I - keyword name       */
            char *comm,        /* I - keyword comment    */
            int *status)       /* IO - error status      */
 {
@@ -111,7 +115,7 @@ int ffukyu(fitsfile *fptr,    /* I - FITS file pointer  */
 }
 /*--------------------------------------------------------------------------*/
 int ffukys(fitsfile *fptr,    /* I - FITS file pointer  */
-           char *keyname,     /* I - keyword name       */
+           const char *keyname,     /* I - keyword name       */
            char *value,       /* I - keyword value      */
            char *comm,        /* I - keyword comment    */
            int *status)       /* IO - error status      */ 
@@ -132,7 +136,7 @@ int ffukys(fitsfile *fptr,    /* I - FITS file pointer  */
 }
 /*--------------------------------------------------------------------------*/
 int ffukls(fitsfile *fptr,    /* I - FITS file pointer  */
-           char *keyname,     /* I - keyword name       */
+           const char *keyname,     /* I - keyword name       */
            char *value,       /* I - keyword value      */
            char *comm,        /* I - keyword comment    */
            int *status)       /* IO - error status      */ 
@@ -154,7 +158,7 @@ int ffukls(fitsfile *fptr,    /* I - FITS file pointer  */
     return(*status);
 }/*--------------------------------------------------------------------------*/
 int ffukyl(fitsfile *fptr,    /* I - FITS file pointer  */
-           char *keyname,     /* I - keyword name       */
+           const char *keyname,     /* I - keyword name       */
            int value,         /* I - keyword value      */
            char *comm,        /* I - keyword comment    */
            int *status)       /* IO - error status      */
@@ -175,8 +179,8 @@ int ffukyl(fitsfile *fptr,    /* I - FITS file pointer  */
 }
 /*--------------------------------------------------------------------------*/
 int ffukyj(fitsfile *fptr,    /* I - FITS file pointer  */
-           char *keyname,     /* I - keyword name       */
-           long value,        /* I - keyword value      */
+           const char *keyname,     /* I - keyword name       */
+           LONGLONG value,    /* I - keyword value      */
            char *comm,        /* I - keyword comment    */
            int *status)       /* IO - error status      */
 {
@@ -196,7 +200,7 @@ int ffukyj(fitsfile *fptr,    /* I - FITS file pointer  */
 }
 /*--------------------------------------------------------------------------*/
 int ffukyf(fitsfile *fptr,    /* I - FITS file pointer  */
-           char *keyname,     /* I - keyword name       */
+           const char *keyname,     /* I - keyword name       */
            float value,       /* I - keyword value      */
            int decim,         /* I - no of decimals     */         
            char *comm,        /* I - keyword comment    */
@@ -218,7 +222,7 @@ int ffukyf(fitsfile *fptr,    /* I - FITS file pointer  */
 }
 /*--------------------------------------------------------------------------*/
 int ffukye(fitsfile *fptr,    /* I - FITS file pointer  */
-           char *keyname,     /* I - keyword name       */
+           const char *keyname,     /* I - keyword name       */
            float value,       /* I - keyword value      */
            int decim,         /* I - no of decimals     */
            char *comm,        /* I - keyword comment    */
@@ -240,7 +244,7 @@ int ffukye(fitsfile *fptr,    /* I - FITS file pointer  */
 }
 /*--------------------------------------------------------------------------*/
 int ffukyg(fitsfile *fptr,    /* I - FITS file pointer  */
-           char *keyname,     /* I - keyword name       */
+           const char *keyname,     /* I - keyword name       */
            double value,      /* I - keyword value      */
            int decim,         /* I - no of decimals     */
            char *comm,        /* I - keyword comment    */
@@ -262,7 +266,7 @@ int ffukyg(fitsfile *fptr,    /* I - FITS file pointer  */
 }
 /*--------------------------------------------------------------------------*/
 int ffukyd(fitsfile *fptr,    /* I - FITS file pointer  */
-           char *keyname,     /* I - keyword name       */
+           const char *keyname,     /* I - keyword name       */
            double value,      /* I - keyword value      */
            int decim,         /* I - no of decimals     */
            char *comm,        /* I - keyword comment    */
@@ -284,7 +288,7 @@ int ffukyd(fitsfile *fptr,    /* I - FITS file pointer  */
 }
 /*--------------------------------------------------------------------------*/
 int ffukfc(fitsfile *fptr,    /* I - FITS file pointer  */
-           char *keyname,     /* I - keyword name       */
+           const char *keyname,     /* I - keyword name       */
            float *value,      /* I - keyword value      */
            int decim,         /* I - no of decimals     */         
            char *comm,        /* I - keyword comment    */
@@ -306,7 +310,7 @@ int ffukfc(fitsfile *fptr,    /* I - FITS file pointer  */
 }
 /*--------------------------------------------------------------------------*/
 int ffukyc(fitsfile *fptr,    /* I - FITS file pointer  */
-           char *keyname,     /* I - keyword name       */
+           const char *keyname,     /* I - keyword name       */
            float *value,      /* I - keyword value      */
            int decim,         /* I - no of decimals     */
            char *comm,        /* I - keyword comment    */
@@ -328,7 +332,7 @@ int ffukyc(fitsfile *fptr,    /* I - FITS file pointer  */
 }
 /*--------------------------------------------------------------------------*/
 int ffukfm(fitsfile *fptr,    /* I - FITS file pointer  */
-           char *keyname,     /* I - keyword name       */
+           const char *keyname,     /* I - keyword name       */
            double *value,     /* I - keyword value      */
            int decim,         /* I - no of decimals     */
            char *comm,        /* I - keyword comment    */
@@ -350,7 +354,7 @@ int ffukfm(fitsfile *fptr,    /* I - FITS file pointer  */
 }
 /*--------------------------------------------------------------------------*/
 int ffukym(fitsfile *fptr,    /* I - FITS file pointer  */
-           char *keyname,     /* I - keyword name       */
+           const char *keyname,     /* I - keyword name       */
            double *value,     /* I - keyword value      */
            int decim,         /* I - no of decimals     */
            char *comm,        /* I - keyword comment    */
@@ -372,7 +376,7 @@ int ffukym(fitsfile *fptr,    /* I - FITS file pointer  */
 }
 /*--------------------------------------------------------------------------*/
 int ffucrd(fitsfile *fptr,    /* I - FITS file pointer  */
-           char *keyname,     /* I - keyword name       */
+           const char *keyname,     /* I - keyword name       */
            char *card,        /* I - card string value  */
            int *status)       /* IO - error status      */
 {
@@ -405,11 +409,12 @@ int ffmrec(fitsfile *fptr,    /* I - FITS file pointer               */
 }
 /*--------------------------------------------------------------------------*/
 int ffmcrd(fitsfile *fptr,    /* I - FITS file pointer  */
-           char *keyname,     /* I - keyword name       */
+           const char *keyname,     /* I - keyword name       */
            char *card,        /* I - card string value  */
            int *status)       /* IO - error status      */
 {
-    char tcard[FLEN_CARD];
+    char tcard[FLEN_CARD], valstring[FLEN_CARD], comm[FLEN_CARD], value[FLEN_CARD];
+    int keypos, len;
 
     if (*status > 0)           /* inherit input status value if > 0 */
         return(*status);
@@ -419,12 +424,33 @@ int ffmcrd(fitsfile *fptr,    /* I - FITS file pointer  */
 
     ffmkey(fptr, card, status);
 
+    /* calc position of keyword in header */
+    keypos = (int) ((((fptr->Fptr)->nextkey) - ((fptr->Fptr)->headstart[(fptr->Fptr)->curhdu])) / 80) + 1;
+
+    ffpsvc(tcard, valstring, comm, status);
+
+    /* check for string value which may be continued over multiple keywords */
+    ffc2s(valstring, value, status);   /* remove quotes and trailing spaces */
+    len = strlen(value);
+
+    while (len && value[len - 1] == '&')  /* ampersand used as continuation char */
+    {
+        ffgcnt(fptr, value, status);
+        if (*value)
+        {
+            ffdrec(fptr, keypos, status);  /* delete the keyword */
+            len = strlen(value);
+        }
+        else   /* a null valstring indicates no continuation */
+            len = 0;
+    }
+
     return(*status);
 }
 /*--------------------------------------------------------------------------*/
 int ffmnam(fitsfile *fptr,    /* I - FITS file pointer     */
-           char *oldname,     /* I - existing keyword name */
-           char *newname,     /* I - new name for keyword  */
+           const char *oldname,     /* I - existing keyword name */
+           const char *newname,     /* I - new name for keyword  */
            int *status)       /* IO - error status         */
 {
     char comm[FLEN_COMMENT];
@@ -444,7 +470,7 @@ int ffmnam(fitsfile *fptr,    /* I - FITS file pointer     */
 }
 /*--------------------------------------------------------------------------*/
 int ffmcom(fitsfile *fptr,    /* I - FITS file pointer  */
-           char *keyname,     /* I - keyword name       */
+           const char *keyname,     /* I - keyword name       */
            char *comm,        /* I - keyword comment    */
            int *status)       /* IO - error status      */
 {
@@ -465,7 +491,7 @@ int ffmcom(fitsfile *fptr,    /* I - FITS file pointer  */
 }
 /*--------------------------------------------------------------------------*/
 int ffpunt(fitsfile *fptr,    /* I - FITS file pointer   */
-           char *keyname,     /* I - keyword name        */
+           const char *keyname,     /* I - keyword name        */
            char *unit,        /* I - keyword unit string */
            int *status)       /* IO - error status       */
 /*
@@ -533,7 +559,7 @@ int ffpunt(fitsfile *fptr,    /* I - FITS file pointer   */
 }
 /*--------------------------------------------------------------------------*/
 int ffmkyu(fitsfile *fptr,    /* I - FITS file pointer  */
-           char *keyname,     /* I - keyword name       */
+           const char *keyname,     /* I - keyword name       */
            char *comm,        /* I - keyword comment    */
            int *status)       /* IO - error status      */
 {
@@ -560,7 +586,7 @@ int ffmkyu(fitsfile *fptr,    /* I - FITS file pointer  */
 }
 /*--------------------------------------------------------------------------*/
 int ffmkys(fitsfile *fptr,    /* I - FITS file pointer  */
-           char *keyname,     /* I - keyword name       */
+           const char *keyname,     /* I - keyword name       */
            char *value,       /* I - keyword value      */
            char *comm,        /* I - keyword comment    */
            int *status)       /* IO - error status      */
@@ -589,7 +615,7 @@ int ffmkys(fitsfile *fptr,    /* I - FITS file pointer  */
 
     ffmkey(fptr, card, status); /* overwrite the previous keyword */
 
-    keypos = ((((fptr->Fptr)->nextkey) - ((fptr->Fptr)->headstart[(fptr->Fptr)->curhdu])) / 80) + 1;
+    keypos = (int) (((((fptr->Fptr)->nextkey) - ((fptr->Fptr)->headstart[(fptr->Fptr)->curhdu])) / 80) + 1);
 
     /* check if old string value was continued over multiple keywords */
     ffc2s(oldval, valstring, status); /* remove quotes and trailing spaces */
@@ -610,7 +636,7 @@ int ffmkys(fitsfile *fptr,    /* I - FITS file pointer  */
 }
 /*--------------------------------------------------------------------------*/
 int ffmkls( fitsfile *fptr,     /* I - FITS file pointer        */
-            char *keyname,      /* I - name of keyword to write */
+            const char *keyname,      /* I - name of keyword to write */
             char *value,        /* I - keyword value            */
             char *incomm,       /* I - keyword comment          */
             int  *status)       /* IO - error status            */
@@ -627,7 +653,7 @@ int ffmkls( fitsfile *fptr,     /* I - FITS file pointer        */
 */
 {
     char valstring[FLEN_VALUE];
-    char card[FLEN_CARD];
+    char card[FLEN_CARD], tmpkeyname[FLEN_CARD];
     char comm[FLEN_COMMENT];
     char tstring[FLEN_VALUE], *cptr;
     char *longval;
@@ -675,7 +701,10 @@ int ffmkls( fitsfile *fptr,     /* I - FITS file pointer        */
         cptr = strchr(cptr, '\'');  /* search for another quote char */
     }
 
-    cptr = keyname;
+    strncpy(tmpkeyname, keyname, 80);
+    tmpkeyname[80] = '\0';
+    
+    cptr = tmpkeyname;
     while(*cptr == ' ')   /* skip over leading spaces in name */
         cptr++;
 
@@ -742,7 +771,7 @@ int ffmkls( fitsfile *fptr,     /* I - FITS file pointer        */
 }
 /*--------------------------------------------------------------------------*/
 int ffmkyl(fitsfile *fptr,    /* I - FITS file pointer  */
-           char *keyname,     /* I - keyword name       */
+           const char *keyname,     /* I - keyword name       */
            int value,         /* I - keyword value      */
            char *comm,        /* I - keyword comment    */
            int *status)       /* IO - error status      */
@@ -769,9 +798,9 @@ int ffmkyl(fitsfile *fptr,    /* I - FITS file pointer  */
     return(*status);
 }
 /*--------------------------------------------------------------------------*/
-int ffmkyj(fitsfile *fptr,    /* I - FITS file pointer  */
-           char *keyname,     /* I - keyword name       */
-           long value,        /* I - keyword value      */
+int ffmkyj(fitsfile *fptr,   /* I - FITS file pointer  */
+           const char *keyname,     /* I - keyword name       */
+           LONGLONG value,    /* I - keyword value      */
            char *comm,        /* I - keyword comment    */
            int *status)       /* IO - error status      */
 {
@@ -798,7 +827,7 @@ int ffmkyj(fitsfile *fptr,    /* I - FITS file pointer  */
 }
 /*--------------------------------------------------------------------------*/
 int ffmkyf(fitsfile *fptr,    /* I - FITS file pointer  */
-           char *keyname,     /* I - keyword name       */
+           const char *keyname,     /* I - keyword name       */
            float value,       /* I - keyword value      */
            int decim,         /* I - no of decimals     */
            char *comm,        /* I - keyword comment    */
@@ -827,7 +856,7 @@ int ffmkyf(fitsfile *fptr,    /* I - FITS file pointer  */
 }
 /*--------------------------------------------------------------------------*/
 int ffmkye(fitsfile *fptr,    /* I - FITS file pointer  */
-           char *keyname,     /* I - keyword name       */
+           const char *keyname,     /* I - keyword name       */
            float value,       /* I - keyword value      */
            int decim,         /* I - no of decimals     */
            char *comm,        /* I - keyword comment    */
@@ -856,7 +885,7 @@ int ffmkye(fitsfile *fptr,    /* I - FITS file pointer  */
 }
 /*--------------------------------------------------------------------------*/
 int ffmkyg(fitsfile *fptr,    /* I - FITS file pointer  */
-           char *keyname,     /* I - keyword name       */
+           const char *keyname,     /* I - keyword name       */
            double value,      /* I - keyword value      */
            int decim,         /* I - no of decimals     */
            char *comm,        /* I - keyword comment    */
@@ -885,7 +914,7 @@ int ffmkyg(fitsfile *fptr,    /* I - FITS file pointer  */
 }
 /*--------------------------------------------------------------------------*/
 int ffmkyd(fitsfile *fptr,    /* I - FITS file pointer  */
-           char *keyname,     /* I - keyword name       */
+           const char *keyname,     /* I - keyword name       */
            double value,      /* I - keyword value      */
            int decim,         /* I - no of decimals     */
            char *comm,        /* I - keyword comment    */
@@ -914,7 +943,7 @@ int ffmkyd(fitsfile *fptr,    /* I - FITS file pointer  */
 }
 /*--------------------------------------------------------------------------*/
 int ffmkfc(fitsfile *fptr,    /* I - FITS file pointer  */
-           char *keyname,     /* I - keyword name       */
+           const char *keyname,     /* I - keyword name       */
            float *value,      /* I - keyword value      */
            int decim,         /* I - no of decimals     */
            char *comm,        /* I - keyword comment    */
@@ -949,7 +978,7 @@ int ffmkfc(fitsfile *fptr,    /* I - FITS file pointer  */
 }
 /*--------------------------------------------------------------------------*/
 int ffmkyc(fitsfile *fptr,    /* I - FITS file pointer  */
-           char *keyname,     /* I - keyword name       */
+           const char *keyname,     /* I - keyword name       */
            float *value,      /* I - keyword value      */
            int decim,         /* I - no of decimals     */
            char *comm,        /* I - keyword comment    */
@@ -984,7 +1013,7 @@ int ffmkyc(fitsfile *fptr,    /* I - FITS file pointer  */
 }
 /*--------------------------------------------------------------------------*/
 int ffmkfm(fitsfile *fptr,    /* I - FITS file pointer  */
-           char *keyname,     /* I - keyword name       */
+           const char *keyname,     /* I - keyword name       */
            double *value,     /* I - keyword value      */
            int decim,         /* I - no of decimals     */
            char *comm,        /* I - keyword comment    */
@@ -1019,7 +1048,7 @@ int ffmkfm(fitsfile *fptr,    /* I - FITS file pointer  */
 }
 /*--------------------------------------------------------------------------*/
 int ffmkym(fitsfile *fptr,    /* I - FITS file pointer  */
-           char *keyname,     /* I - keyword name       */
+           const char *keyname,     /* I - keyword name       */
            double *value,     /* I - keyword value      */
            int decim,         /* I - no of decimals     */
            char *comm,        /* I - keyword comment    */
@@ -1054,7 +1083,7 @@ int ffmkym(fitsfile *fptr,    /* I - FITS file pointer  */
 }
 /*--------------------------------------------------------------------------*/
 int ffikyu(fitsfile *fptr,    /* I - FITS file pointer  */
-           char *keyname,     /* I - keyword name       */
+           const char *keyname,     /* I - keyword name       */
            char *comm,        /* I - keyword comment    */
            int *status)       /* IO - error status      */
 /*
@@ -1075,7 +1104,7 @@ int ffikyu(fitsfile *fptr,    /* I - FITS file pointer  */
 }
 /*--------------------------------------------------------------------------*/
 int ffikys(fitsfile *fptr,    /* I - FITS file pointer  */
-           char *keyname,     /* I - keyword name       */
+           const char *keyname,     /* I - keyword name       */
            char *value,       /* I - keyword value      */
            char *comm,        /* I - keyword comment    */
            int *status)       /* IO - error status      */
@@ -1094,7 +1123,7 @@ int ffikys(fitsfile *fptr,    /* I - FITS file pointer  */
 }
 /*--------------------------------------------------------------------------*/
 int ffikls( fitsfile *fptr,     /* I - FITS file pointer        */
-            char *keyname,      /* I - name of keyword to write */
+            const char *keyname,      /* I - name of keyword to write */
             char *value,        /* I - keyword value            */
             char *comm,         /* I - keyword comment          */
             int  *status)       /* IO - error status            */
@@ -1108,7 +1137,7 @@ int ffikls( fitsfile *fptr,     /* I - FITS file pointer        */
 */
 {
     char valstring[FLEN_VALUE];
-    char card[FLEN_CARD];
+    char card[FLEN_CARD], tmpkeyname[FLEN_CARD];
     char tstring[FLEN_VALUE], *cptr;
     int next, remain, vlen, nquote, nchar, namelen, contin, tstatus = -1;
 
@@ -1130,7 +1159,11 @@ int ffikls( fitsfile *fptr,     /* I - FITS file pointer        */
         cptr = strchr(cptr, '\'');  /* search for another quote char */
     }
 
-    cptr = keyname;
+
+    strncpy(tmpkeyname, keyname, 80);
+    tmpkeyname[80] = '\0';
+    
+    cptr = tmpkeyname;
     while(*cptr == ' ')   /* skip over leading spaces in name */
         cptr++;
 
@@ -1196,7 +1229,7 @@ int ffikls( fitsfile *fptr,     /* I - FITS file pointer        */
 }
 /*--------------------------------------------------------------------------*/
 int ffikyl(fitsfile *fptr,    /* I - FITS file pointer  */
-           char *keyname,     /* I - keyword name       */
+           const char *keyname,     /* I - keyword name       */
            int value,         /* I - keyword value      */
            char *comm,        /* I - keyword comment    */
            int *status)       /* IO - error status      */
@@ -1215,8 +1248,8 @@ int ffikyl(fitsfile *fptr,    /* I - FITS file pointer  */
 }
 /*--------------------------------------------------------------------------*/
 int ffikyj(fitsfile *fptr,    /* I - FITS file pointer  */
-           char *keyname,     /* I - keyword name       */
-           long value,        /* I - keyword value      */
+           const char *keyname,     /* I - keyword name       */
+           LONGLONG value,    /* I - keyword value      */
            char *comm,        /* I - keyword comment    */
            int *status)       /* IO - error status      */
 {
@@ -1234,7 +1267,7 @@ int ffikyj(fitsfile *fptr,    /* I - FITS file pointer  */
 }
 /*--------------------------------------------------------------------------*/
 int ffikyf(fitsfile *fptr,    /* I - FITS file pointer  */
-           char *keyname,     /* I - keyword name       */
+           const char *keyname,     /* I - keyword name       */
            float value,       /* I - keyword value      */
            int decim,         /* I - no of decimals     */
            char *comm,        /* I - keyword comment    */ 
@@ -1254,7 +1287,7 @@ int ffikyf(fitsfile *fptr,    /* I - FITS file pointer  */
 }
 /*--------------------------------------------------------------------------*/
 int ffikye(fitsfile *fptr,    /* I - FITS file pointer  */
-           char *keyname,     /* I - keyword name       */
+           const char *keyname,     /* I - keyword name       */
            float value,       /* I - keyword value      */
            int decim,         /* I - no of decimals     */
            char *comm,        /* I - keyword comment    */ 
@@ -1274,7 +1307,7 @@ int ffikye(fitsfile *fptr,    /* I - FITS file pointer  */
 }
 /*--------------------------------------------------------------------------*/
 int ffikyg(fitsfile *fptr,    /* I - FITS file pointer  */
-           char *keyname,     /* I - keyword name       */
+           const char *keyname,     /* I - keyword name       */
            double value,      /* I - keyword value      */
            int decim,         /* I - no of decimals     */
            char *comm,        /* I - keyword comment    */ 
@@ -1294,7 +1327,7 @@ int ffikyg(fitsfile *fptr,    /* I - FITS file pointer  */
 }
 /*--------------------------------------------------------------------------*/
 int ffikyd(fitsfile *fptr,    /* I - FITS file pointer  */
-           char *keyname,     /* I - keyword name       */
+           const char *keyname,     /* I - keyword name       */
            double value,      /* I - keyword value      */
            int decim,         /* I - no of decimals     */
            char *comm,        /* I - keyword comment    */ 
@@ -1314,7 +1347,7 @@ int ffikyd(fitsfile *fptr,    /* I - FITS file pointer  */
 }
 /*--------------------------------------------------------------------------*/
 int ffikfc(fitsfile *fptr,    /* I - FITS file pointer  */
-           char *keyname,     /* I - keyword name       */
+           const char *keyname,     /* I - keyword name       */
            float *value,      /* I - keyword value      */
            int decim,         /* I - no of decimals     */
            char *comm,        /* I - keyword comment    */ 
@@ -1341,7 +1374,7 @@ int ffikfc(fitsfile *fptr,    /* I - FITS file pointer  */
 }
 /*--------------------------------------------------------------------------*/
 int ffikyc(fitsfile *fptr,    /* I - FITS file pointer  */
-           char *keyname,     /* I - keyword name       */
+           const char *keyname,     /* I - keyword name       */
            float *value,      /* I - keyword value      */
            int decim,         /* I - no of decimals     */
            char *comm,        /* I - keyword comment    */ 
@@ -1368,7 +1401,7 @@ int ffikyc(fitsfile *fptr,    /* I - FITS file pointer  */
 }
 /*--------------------------------------------------------------------------*/
 int ffikfm(fitsfile *fptr,    /* I - FITS file pointer  */
-           char *keyname,     /* I - keyword name       */
+           const char *keyname,     /* I - keyword name       */
            double *value,     /* I - keyword value      */
            int decim,         /* I - no of decimals     */
            char *comm,        /* I - keyword comment    */ 
@@ -1396,7 +1429,7 @@ int ffikfm(fitsfile *fptr,    /* I - FITS file pointer  */
 }
 /*--------------------------------------------------------------------------*/
 int ffikym(fitsfile *fptr,    /* I - FITS file pointer  */
-           char *keyname,     /* I - keyword name       */
+           const char *keyname,     /* I - keyword name       */
            double *value,     /* I - keyword value      */
            int decim,         /* I - no of decimals     */
            char *comm,        /* I - keyword comment    */ 
@@ -1445,7 +1478,7 @@ int ffikey(fitsfile *fptr,    /* I - FITS file pointer  */
 {
     int ii, len, nshift;
     long nblocks;
-    OFF_T bytepos;
+    LONGLONG bytepos;
     char *inbuff, *outbuff, *tmpbuff, buff1[FLEN_CARD], buff2[FLEN_CARD];
 
     if (*status > 0)           /* inherit input status value if > 0 */
@@ -1462,7 +1495,8 @@ int ffikey(fitsfile *fptr,    /* I - FITS file pointer  */
             return(*status);  
     }
 
-    nshift=( (fptr->Fptr)->headend - (fptr->Fptr)->nextkey ) / 80; /* no. keywords to shift */
+    /* no. keywords to shift */
+    nshift= (int) (( (fptr->Fptr)->headend - (fptr->Fptr)->nextkey ) / 80); 
 
     strncpy(buff2, card, 80);     /* copy card to output buffer */
     buff2[80] = '\0';
@@ -1506,7 +1540,7 @@ int ffikey(fitsfile *fptr,    /* I - FITS file pointer  */
 }
 /*--------------------------------------------------------------------------*/
 int ffdkey(fitsfile *fptr,    /* I - FITS file pointer  */
-           char *keyname,     /* I - keyword name       */
+           const char *keyname,     /* I - keyword name       */
            int *status)       /* IO - error status      */
 /*
   delete a specified header keyword
@@ -1528,7 +1562,7 @@ int ffdkey(fitsfile *fptr,    /* I - FITS file pointer  */
     }
 
     /* calc position of keyword in header */
-    keypos = (((fptr->Fptr)->nextkey) - ((fptr->Fptr)->headstart[(fptr->Fptr)->curhdu])) / 80;
+    keypos = (int) ((((fptr->Fptr)->nextkey) - ((fptr->Fptr)->headstart[(fptr->Fptr)->curhdu])) / 80);
 
     ffdrec(fptr, keypos, status);  /* delete the keyword */
 
@@ -1558,7 +1592,7 @@ int ffdrec(fitsfile *fptr,   /* I - FITS file pointer  */
 */
 {
     int ii, nshift;
-    OFF_T bytepos;
+    LONGLONG bytepos;
     char *inbuff, *outbuff, *tmpbuff, buff1[81], buff2[81];
     char message[FLEN_ERRMSG];
 
@@ -1575,7 +1609,7 @@ int ffdrec(fitsfile *fptr,   /* I - FITS file pointer  */
 
     (fptr->Fptr)->nextkey = (fptr->Fptr)->headstart[(fptr->Fptr)->curhdu] + (keypos - 1) * 80;
 
-    nshift=( (fptr->Fptr)->headend - (fptr->Fptr)->nextkey ) / 80; /* no. keywords to shift */
+    nshift=(int) (( (fptr->Fptr)->headend - (fptr->Fptr)->nextkey ) / 80); /* no. keywords to shift */
 
     if (nshift <= 0)
     {
