@@ -55,6 +55,10 @@ static int FnCompare_short (const void *, const void *);
 static int FnCompare_int (const void *, const void *);
 static int FnCompare_float (const void *, const void *);
 static int FnCompare_double (const void *, const void *);
+static float quick_select_float(float arr[], int n);
+static short quick_select_short(short arr[], int n);
+static int quick_select_int(int arr[], int n);
+static double quick_select_double(double arr[], int n);
 /*---------------------------------------------------------------------------*/
 int fits_quantize_float (float fdata[], long nxpix, long nypix, int nullcheck, 
 	float in_null_value, float qlevel, int idata[], double *bscale,
@@ -115,7 +119,7 @@ If the function value is zero, the data were not copied to idata.
 		break;
 	    }
             else {
-  	        itemp = (int)(fdata[i] + 0.5);
+  	        itemp = (int)(fdata[i] + 0.5f);
 
 	        if (itemp != fdata[i]) {
 		    intflag = 0;	/* not integer */
@@ -221,8 +225,7 @@ If the function value is zero, the data were not copied to idata.
             }
 
        	    for (i = 0;  i < nx;  i++) {
-	        temp = (fdata[i] - zeropt) / delta;
-	        idata[i] = NINT (temp);
+	        idata[i] = NINT ((fdata[i] - zeropt) / delta);
             }
         }
         else {
@@ -232,8 +235,7 @@ If the function value is zero, the data were not copied to idata.
 
 	    for (i = 0;  i < nx;  i++) {
                 if (fdata[i] != in_null_value) {
-	            temp = (fdata[i] - zeropt) / delta;
-	            idata[i] = NINT (temp);
+	            idata[i] = NINT ((fdata[i] - zeropt) / delta);
                 }
                 else
                     idata[i] = NULL_VALUE;
@@ -1018,8 +1020,8 @@ row of the image.
 		} else if (nvals == 1) {
 		    diffs[nrows] = differences[0];
 		} else {
-	            qsort(differences, nvals, sizeof(short), FnCompare_short);
-	            diffs[nrows] = (differences[(nvals - 1) / 2] + differences[(nvals) / 2]) / 2.;
+                    /* quick_select returns the median MUCH faster than using qsort */
+                    diffs[nrows] = quick_select_short(differences, nvals);
 		}
 
 		nrows++;
@@ -1235,8 +1237,8 @@ row of the image.
 		} else if (nvals == 1) {
 		    diffs[nrows] = differences[0];
 		} else {
-	            qsort(differences, nvals, sizeof(int), FnCompare_int);
-	            diffs[nrows] = (differences[(nvals - 1) / 2] + differences[(nvals) / 2]) / 2.;
+                    /* quick_select returns the median MUCH faster than using qsort */
+                    diffs[nrows] = quick_select_int(differences, nvals);
 		}
 
 		nrows++;
@@ -1317,7 +1319,7 @@ row of the image.
 		ny = 1;
 	}
 
-	/* rows must have at least 5 pixels */
+	/* rows must have at least 5 pixels to calc noise, so just calc min, max, ngood */
 	if (nx < 5) {
 
 		for (ii = 0; ii < nx; ii++) {
@@ -1458,8 +1460,8 @@ row of the image.
 		    } else if (nvals == 1) {
 		        diffs[nrows] = differences[0];
 		    } else {
-	                qsort(differences, nvals, sizeof(float), FnCompare_float);
-	                diffs[nrows] = (differences[(nvals - 1) / 2] + differences[(nvals) / 2]) / 2.;
+                        /* quick_select returns the median MUCH faster than using qsort */
+                        diffs[nrows] = quick_select_float(differences, nvals);
 		    }
 		}
 		nrows++;
@@ -1668,8 +1670,8 @@ row of the image.
 		    } else if (nvals == 1) {
 		        diffs[nrows] = differences[0];
 		    } else {
-	                qsort(differences, nvals, sizeof(double), FnCompare_double);
-	                diffs[nrows] = (differences[(nvals - 1) / 2] + differences[(nvals) / 2]) / 2.;
+                        /* quick_select returns the median MUCH faster than using qsort */
+                        diffs[nrows] = quick_select_double(differences, nvals);
 		    }
 		}
 		nrows++;
@@ -1720,7 +1722,7 @@ row of the image.
 	int iter;
 	long ii, jj, kk, nrows = 0, nvals;
 	short *differences, *rowpix, v1;
-	double  *diffs, xnoise = 0., mean, stdev;
+	double  *diffs, xnoise, mean, stdev;
 
 	/* rows must have at least 3 pixels to estimate noise */
 	if (nx < 3) {
@@ -1841,7 +1843,7 @@ row of the image.
 	int iter;
 	long ii, jj, kk, nrows = 0, nvals;
 	int *differences, *rowpix, v1;
-	double  *diffs, xnoise = 0., mean, stdev;
+	double  *diffs, xnoise, mean, stdev;
 
 	/* rows must have at least 3 pixels to estimate noise */
 	if (nx < 3) {
@@ -1962,7 +1964,7 @@ row of the image.
 	int iter;
 	long ii, jj, kk, nrows = 0, nvals;
 	float *differences, *rowpix, v1;
-	double  *diffs, xnoise = 0., mean, stdev;
+	double  *diffs, xnoise, mean, stdev;
 
 	/* rows must have at least 3 pixels to estimate noise */
 	if (nx < 3) {
@@ -2083,7 +2085,7 @@ row of the image.
 	int iter;
 	long ii, jj, kk, nrows = 0, nvals;
 	double *differences, *rowpix, v1;
-	double  *diffs, xnoise = 0., mean, stdev;
+	double  *diffs, xnoise, mean, stdev;
 
 	/* rows must have at least 3 pixels to estimate noise */
 	if (nx < 3) {
@@ -2234,3 +2236,237 @@ static int FnCompare_double(const void *v1, const void *v2)
    else
      return(0);
 }
+/*--------------------------------------------------------------------------*/
+
+/*
+ *  These Quickselect routines are based on the algorithm described in
+ *  "Numerical recipes in C", Second Edition,
+ *  Cambridge University Press, 1992, Section 8.5, ISBN 0-521-43108-5
+ *  This code by Nicolas Devillard - 1998. Public domain.
+ */
+
+/*--------------------------------------------------------------------------*/
+
+#define ELEM_SWAP(a,b) { register float t=(a);(a)=(b);(b)=t; }
+
+static float quick_select_float(float arr[], int n) 
+{
+    int low, high ;
+    int median;
+    int middle, ll, hh;
+
+    low = 0 ; high = n-1 ; median = (low + high) / 2;
+    for (;;) {
+        if (high <= low) /* One element only */
+            return arr[median] ;
+
+        if (high == low + 1) {  /* Two elements only */
+            if (arr[low] > arr[high])
+                ELEM_SWAP(arr[low], arr[high]) ;
+            return arr[median] ;
+        }
+
+    /* Find median of low, middle and high items; swap into position low */
+    middle = (low + high) / 2;
+    if (arr[middle] > arr[high])    ELEM_SWAP(arr[middle], arr[high]) ;
+    if (arr[low] > arr[high])       ELEM_SWAP(arr[low], arr[high]) ;
+    if (arr[middle] > arr[low])     ELEM_SWAP(arr[middle], arr[low]) ;
+
+    /* Swap low item (now in position middle) into position (low+1) */
+    ELEM_SWAP(arr[middle], arr[low+1]) ;
+
+    /* Nibble from each end towards middle, swapping items when stuck */
+    ll = low + 1;
+    hh = high;
+    for (;;) {
+        do ll++; while (arr[low] > arr[ll]) ;
+        do hh--; while (arr[hh]  > arr[low]) ;
+
+        if (hh < ll)
+        break;
+
+        ELEM_SWAP(arr[ll], arr[hh]) ;
+    }
+
+    /* Swap middle item (in position low) back into correct position */
+    ELEM_SWAP(arr[low], arr[hh]) ;
+
+    /* Re-set active partition */
+    if (hh <= median)
+        low = ll;
+        if (hh >= median)
+        high = hh - 1;
+    }
+}
+
+#undef ELEM_SWAP
+
+/*--------------------------------------------------------------------------*/
+
+#define ELEM_SWAP(a,b) { register short t=(a);(a)=(b);(b)=t; }
+
+static short quick_select_short(short arr[], int n) 
+{
+    int low, high ;
+    int median;
+    int middle, ll, hh;
+
+    low = 0 ; high = n-1 ; median = (low + high) / 2;
+    for (;;) {
+        if (high <= low) /* One element only */
+            return arr[median] ;
+
+        if (high == low + 1) {  /* Two elements only */
+            if (arr[low] > arr[high])
+                ELEM_SWAP(arr[low], arr[high]) ;
+            return arr[median] ;
+        }
+
+    /* Find median of low, middle and high items; swap into position low */
+    middle = (low + high) / 2;
+    if (arr[middle] > arr[high])    ELEM_SWAP(arr[middle], arr[high]) ;
+    if (arr[low] > arr[high])       ELEM_SWAP(arr[low], arr[high]) ;
+    if (arr[middle] > arr[low])     ELEM_SWAP(arr[middle], arr[low]) ;
+
+    /* Swap low item (now in position middle) into position (low+1) */
+    ELEM_SWAP(arr[middle], arr[low+1]) ;
+
+    /* Nibble from each end towards middle, swapping items when stuck */
+    ll = low + 1;
+    hh = high;
+    for (;;) {
+        do ll++; while (arr[low] > arr[ll]) ;
+        do hh--; while (arr[hh]  > arr[low]) ;
+
+        if (hh < ll)
+        break;
+
+        ELEM_SWAP(arr[ll], arr[hh]) ;
+    }
+
+    /* Swap middle item (in position low) back into correct position */
+    ELEM_SWAP(arr[low], arr[hh]) ;
+
+    /* Re-set active partition */
+    if (hh <= median)
+        low = ll;
+        if (hh >= median)
+        high = hh - 1;
+    }
+}
+
+#undef ELEM_SWAP
+
+/*--------------------------------------------------------------------------*/
+
+#define ELEM_SWAP(a,b) { register int t=(a);(a)=(b);(b)=t; }
+
+static int quick_select_int(int arr[], int n) 
+{
+    int low, high ;
+    int median;
+    int middle, ll, hh;
+
+    low = 0 ; high = n-1 ; median = (low + high) / 2;
+    for (;;) {
+        if (high <= low) /* One element only */
+            return arr[median] ;
+
+        if (high == low + 1) {  /* Two elements only */
+            if (arr[low] > arr[high])
+                ELEM_SWAP(arr[low], arr[high]) ;
+            return arr[median] ;
+        }
+
+    /* Find median of low, middle and high items; swap into position low */
+    middle = (low + high) / 2;
+    if (arr[middle] > arr[high])    ELEM_SWAP(arr[middle], arr[high]) ;
+    if (arr[low] > arr[high])       ELEM_SWAP(arr[low], arr[high]) ;
+    if (arr[middle] > arr[low])     ELEM_SWAP(arr[middle], arr[low]) ;
+
+    /* Swap low item (now in position middle) into position (low+1) */
+    ELEM_SWAP(arr[middle], arr[low+1]) ;
+
+    /* Nibble from each end towards middle, swapping items when stuck */
+    ll = low + 1;
+    hh = high;
+    for (;;) {
+        do ll++; while (arr[low] > arr[ll]) ;
+        do hh--; while (arr[hh]  > arr[low]) ;
+
+        if (hh < ll)
+        break;
+
+        ELEM_SWAP(arr[ll], arr[hh]) ;
+    }
+
+    /* Swap middle item (in position low) back into correct position */
+    ELEM_SWAP(arr[low], arr[hh]) ;
+
+    /* Re-set active partition */
+    if (hh <= median)
+        low = ll;
+        if (hh >= median)
+        high = hh - 1;
+    }
+}
+
+#undef ELEM_SWAP
+
+/*--------------------------------------------------------------------------*/
+
+#define ELEM_SWAP(a,b) { register double t=(a);(a)=(b);(b)=t; }
+
+static double quick_select_double(double arr[], int n) 
+{
+    int low, high ;
+    int median;
+    int middle, ll, hh;
+
+    low = 0 ; high = n-1 ; median = (low + high) / 2;
+    for (;;) {
+        if (high <= low) /* One element only */
+            return arr[median] ;
+
+        if (high == low + 1) {  /* Two elements only */
+            if (arr[low] > arr[high])
+                ELEM_SWAP(arr[low], arr[high]) ;
+            return arr[median] ;
+        }
+
+    /* Find median of low, middle and high items; swap into position low */
+    middle = (low + high) / 2;
+    if (arr[middle] > arr[high])    ELEM_SWAP(arr[middle], arr[high]) ;
+    if (arr[low] > arr[high])       ELEM_SWAP(arr[low], arr[high]) ;
+    if (arr[middle] > arr[low])     ELEM_SWAP(arr[middle], arr[low]) ;
+
+    /* Swap low item (now in position middle) into position (low+1) */
+    ELEM_SWAP(arr[middle], arr[low+1]) ;
+
+    /* Nibble from each end towards middle, swapping items when stuck */
+    ll = low + 1;
+    hh = high;
+    for (;;) {
+        do ll++; while (arr[low] > arr[ll]) ;
+        do hh--; while (arr[hh]  > arr[low]) ;
+
+        if (hh < ll)
+        break;
+
+        ELEM_SWAP(arr[ll], arr[hh]) ;
+    }
+
+    /* Swap middle item (in position low) back into correct position */
+    ELEM_SWAP(arr[low], arr[hh]) ;
+
+    /* Re-set active partition */
+    if (hh <= median)
+        low = ll;
+        if (hh >= median)
+        high = hh - 1;
+    }
+}
+
+#undef ELEM_SWAP
+
+
